#include "client.h"

struct {
  struct sockaddr_un server_addr;
  socklen_t server_addr_len;
  int sock_fd;
  char* response;
} client_globals = {
  .response = NULL,
};

int client_init(char* path) {
  struct sockaddr_un client_addr;
  // if we specify this as the size, the socket will get a unique abstract autobind name
  socklen_t client_addr_len = sizeof(sa_family_t);

  client_globals.sock_fd = socket(AF_UNIX, SOCK_DGRAM, 0);
  if (client_globals.sock_fd < 0) {
    return errno; // TODO make an error code enum
  }

  memset(&client_addr, 0, sizeof(client_addr));
  client_addr.sun_family = AF_UNIX;

  if (bind(client_globals.sock_fd, (struct sockaddr *)&client_addr, client_addr_len) != 0) {
    return errno; // TODO make an error code enum
  }

  // set up server address for all sockets to use
  memset(&client_globals.server_addr, 0, sizeof(client_globals.server_addr));
  client_globals.server_addr.sun_family = AF_UNIX;
  // copy in the provided directory path for the socket
  memcpy(client_globals.server_addr.sun_path, path, strlen(path));

  client_globals.server_addr_len = sizeof(client_globals.server_addr.sun_family) + strlen(client_globals.server_addr.sun_path);

  return 0;
}

int client_request(char* request) {
  sendto(client_globals.sock_fd, request, strlen(request) + 1, 0, (struct sockaddr *) &client_globals.server_addr, client_globals.server_addr_len);

  // RECEIVE
  ssize_t msg_size;

  {
    // make copy of server address struct
    struct sockaddr_un server_addr = server_addr;
    // recvfrom requires a pointer to server_addr_len, so make a copy just in case it does something to it
    socklen_t server_addr_len_copy = client_globals.server_addr_len;

    msg_size = recvfrom(
      client_globals.sock_fd,
      NULL,
      0,
      MSG_PEEK | MSG_TRUNC,
      (struct sockaddr *) &server_addr,
      &server_addr_len_copy
    );
    if (msg_size < 0) {
      return errno;
    }
  }

  {
    free(client_globals.response);
    client_globals.response = malloc(msg_size);

    // make a new copy of server address struct
    struct sockaddr_un server_addr = server_addr;
    // recvfrom requires a pointer to server_addr_len, so make a copy just in case it does something to it
    socklen_t server_addr_len_copy = client_globals.server_addr_len;

    ssize_t recv_size;
    recv_size = recvfrom(
      client_globals.sock_fd,
      client_globals.response,
      msg_size,
      0,
      (struct sockaddr *) &server_addr,
      &server_addr_len_copy
    );

    if (recv_size < 0) {
      return errno;
    }
  }

  return 0;
}

char* client_get_response() {
  return client_globals.response;
}

// void __attribute__ ((constructor)) init(void) {

// }

// TODO: proper signal handling (e.g. this doesn't run when SIGINT stops the program)
void __attribute__ ((destructor)) client_clean_up(void) {
  free(client_globals.response);
  close(client_globals.sock_fd);
}
