#include "localserv.h"

int main(int argc, char *argv[]) {
  interrupt_register_sighandler();
  set_exe_name(argv[0]);

  // OSSL_DECODER_CTX_new_for_pkey()
  // BIO_new_file()
  // OSSL_DECODER_from_bio()

  if (argc <= 1) { // no subcommand!
    fprintf(stderr, "Please provide a subcommand.\n\n");
    interrupt_set_exit_flag();
  }
  else {
    opts opts = {
      .help = 0,
      .username = 0,
      .auth = 0,
      .delims = { '\t', '\n', '\n' },
    };

    static struct option help_opt[] = {
      {"help",      no_argument, NULL, 'h'},
      {"username",  no_argument, NULL, 'u'},
      {"auth",      no_argument, NULL, 'a'},
      // TODO: args for delims
      {0, 0, 0, 0},
    };
    int option_index = 0;
    // // Configures getopt to not print its own error message
    // opterr = 0;
    int optchar;
    while ((optchar = getopt_long(argc, argv, "hua", help_opt, &option_index)) != -1) {
      switch (optchar) {
        case 'h':
          opts.help = 1;
          break;
        case 'u':
          opts.username = 1;
          break;
        case 'a':
          opts.auth = 1;
          // authmode implies username
          opts.username = 1;
          break;
        // TODO: delims
        default:
          interrupt_set_exit_flag();
          break;
      }
    }

    if (!interrupt_should_exit()) {
      if (optind < argc) {
        char *subcommand = argv[optind];
        if (strcmp(subcommand, "server") == 0) {
          server_subcommand(opts, argc - optind, argv + optind);
        }
        else if (strcmp(subcommand, "client") == 0) {
          client_subcommand(opts, argc - optind, argv + optind);
        }
        else {
          fprintf(stderr, "Subcommand “%s” not recognized; provide a valid subcommand as the first argument.\n\n", subcommand);
          show_usage(true);
          interrupt_set_exit_flag();
        }
      }
      else if (opts.help) {
        show_usage(false);
      }
      else {
        // if optind is equal to (or greater than, which probably won't happen) the number of args we got,
        // it means we only got options, no positional args
        fprintf(stderr, "Please provide a subcommand.\n\n");
        show_usage(true);
        interrupt_set_exit_flag();
      }
    }
  }

  return interrupt_return_value();
}
