#include "auth.h"

#define ISSUE "ISSUE"
#define REFRESH "REFRESH\n"
#define REFRESH_OFFSET sizeof(REFRESH) - 1
#define AUTHORIZE "AUTHORIZE\n"
#define AUTHORIZE_OFFSET sizeof(AUTHORIZE) - 1
#define ISSUE_EXPIRATION 15
#define REFRESH_EXPIRATION 1200
#define grant_failure(grant) fprintf(stderr, "JWT %s set failed: %s\n", grant, strerror(error))

unsigned char *key = (unsigned char *)"THIS IS NOT A SAFE KEY! -- TODO: use a user-provided key instead";
int keylen = 64;

int set_jwt_grants(jwt_t *jwt, const char *sub, const time_t exp) {
  int error = jwt_set_alg(jwt, JWT_ALG_HS256, key, keylen);
  if (error) {
    grant_failure("alg");
    return error;
  }

  error = jwt_add_grant(jwt, "sub", sub);
  if (error) {
    grant_failure("sub");
    return error;
  }
  
  time_t iat = time(NULL);
  error = jwt_add_grant_int(jwt, "iat", iat);
  if (error) {
    grant_failure("iat");
    return error;
  }
  
  error = jwt_add_grant_int(jwt, "exp", iat + exp);
  if (error) {
    grant_failure("exp");
    return error;
  }

  return 0;
}

char* auth_issue_jwt(const char *sub, const time_t exp) {
  jwt_t *jwt = NULL;

  int error = jwt_new(&jwt);
  char *response = NULL;
  if (error) {
    fprintf(stderr, "JWT initialization failed: %s\n", strerror(error));
    return NULL;
  }
  error = set_jwt_grants(jwt, sub, exp);
  if (error) {
    jwt_free(jwt);
    return NULL;
  }

  response = jwt_encode_str(jwt);
  jwt_free(jwt);
  if (response == NULL) {
    fprintf(stderr, "failed emitting JWT: %s\n", strerror(errno));
  }
  return response;
}

char* auth_verify_jwt(const char *jwt_str, time_t now, char **sub) {
  jwt_t *jwt;
  int error = jwt_decode(&jwt, jwt_str, key, keylen);
  if (error) {
    fprintf(stderr, "JWT validation failed: %s\n", strerror(error));
    // TODO: determine which errnos correspond to invalid credentials
    // and which correspond to server errors
    // and then return the right thing here
    return strdup("Bad Credential");
  }
  time_t exp = jwt_get_grant_int(jwt, "exp");
  if (errno == ENOENT) {
    return strdup("Bad Credential");
  }
  else if (now >= exp) {
    return strdup("Token Expired");
  }
  *sub = jwt_get_grant(jwt, "sub");
  if (sub == NULL) {
    return strdup("Bad Credential");
  }
  return NULL;
}

char* auth(char **request, char **username) {
  time_t now = time(NULL);
  if (strcasecmp(*request, ISSUE) == 0) {
    char *response = auth_issue_jwt(*username, ISSUE_EXPIRATION);
    return response == NULL ? strdup("Internal Server Error") : response;
  }
  else if (strncasecmp(*request, REFRESH, REFRESH_OFFSET) == 0) {
    char *response = auth_verify_jwt(*request + REFRESH_OFFSET, now, username);
    if (response == NULL) {
      if (*username == NULL) {
        return strdup("Internal Server Error");
      }
      response = auth_issue_jwt(*username, REFRESH_EXPIRATION);
    }
    return response == NULL ? strdup("Internal Server Error") : response;
  }
  else if (strncasecmp(*request, AUTHORIZE, AUTHORIZE_OFFSET) == 0) {
    char *newline = strchr(*request + AUTHORIZE_OFFSET, '\n');
    if (newline != NULL) {
      // usually there should be a message after the token.
      // if so, we replace this newline with a null char so JWT parsing works
      *newline = '\0';
    }
    char *response = auth_verify_jwt(*request + AUTHORIZE_OFFSET, now, username);
    if (*username == NULL) {
      return strdup("Internal Server Error");
    }
    // response NULL means we verified!
    // pass along request by moving past auth stuff
    if (response == NULL) {
      *request = newline == NULL ? "" : newline + 1;
    }
    return response;
  }
  return strdup("Bad Request");
}
