#include "server.h"

void server(char *path, opts *opts) {
  int error;
  error = server_init(path);
  if (error) {
    fprintf(stderr, "server init failed: %s\n", strerror(error));
    interrupt_set_exit_flag();
  }

  int listen_fails = 0;
  char* response = NULL;

  while (!interrupt_should_exit()) {
    error = server_listen();
    if (error) {
      if (interrupt_should_exit()) {
        // no message necessary
        break;
      }
      fprintf(stderr, "listen failed: %s\n", strerror(error));
      if (++listen_fails > 5) {
        fprintf(stderr, "listen failed 5 times, exiting\n");
        interrupt_set_exit_flag();
        break;
      }
      continue;
    }

    // figure out username
    char *username = NULL;
    if ((*opts).username) {
      username = server_get_username(server_get_uid());
      if (username == NULL) {
        fprintf(stderr, "failed getting username\n");
        interrupt_set_exit_flag();
        break;
      }
    }


    char *request = server_get_request();
    if ((*opts).auth) {
      response = auth(&request, &username);
    }

    // if auth supplied us with a response,
    // no need to do the interactive loop
    if (response == NULL) {
      // write request to stdout
      if (username) {
        printf("%s%c", username, (*opts).delims[0]);
      }
      printf("%s%c", request, (*opts).delims[1]);
      fflush(stdout);

      // read user input
      size_t len = 0;
      ssize_t length = getdelim(&response, &len, (*opts).delims[2], stdin);
      if (length == -1) {
        fprintf(stderr, "failed getting input\n");
        interrupt_set_exit_flag();
        break;
      }
      response[length - 1] = '\0';
    }

    error = server_respond(response);

    if (error) {
      fprintf(stderr, "respond failed (will retry): %s\n", strerror(error));
      error = server_respond(response);
      if (error) {
        fprintf(stderr, "respond retry failed: %s\nfailed retry, exiting\n", strerror(error));
        interrupt_set_exit_flag();
        break;
      }
    }
    free(response);
    response = NULL;
  }
  free(response);
}

void server_subcommand(opts opts, int argc, char *argv[]) {
  set_usage_context(usage_server);

  if (opts.help) {
    show_usage(false);
  }
  else if (argc <= 1) {
    fprintf(stderr, "server: please provide a path\n\n");
    show_usage(true);
    interrupt_set_exit_flag();
  }
  else {
    char *path = argv[1];
    server(path, &opts);
  }
}
